// Copyright (C) 2015 Ryan Curtin
//
// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0. If a copy of the MPL was not distributed with this
// file, You can obtain one at http://mozilla.org/MPL/2.0/.



#if defined(ARMA_USE_SUPERLU)

//! \namespace superlu namespace for SuperLU functions
namespace superlu
  {

  // Note that in these functions, sluMatType will be SuperMatrix,
  // slu_c::SuperMatrix, slu_z::SuperMatrix, or slu_s::SuperMatrix.  Other
  // templated types exist for the same reason.  The double versions are in the
  // global namespace; see include_superlu.hpp (try not to cry).

  template<typename eT>
  inline
  void
  gssv(superlu_options_t* options, SuperMatrix* A, int* perm_c, int* perm_r, SuperMatrix* L, SuperMatrix* U, SuperMatrix* B, SuperLUStat_t* stat, int* info)
    {
    arma_type_check( is_supported_blas_type<eT>::value == false );

    if (is_float<eT>::value == true)
      {
      // Call sgssv() overload.
      slu_s::sgssv(options, A, perm_c, perm_r, L, U, B, stat, info);
      }
    else
    if (is_double<eT>::value == true)
      {
      // Call dgssv() overload.  It's not in any namespace.
      dgssv(options, A, perm_c, perm_r, L, U, B, stat, info);
      }
    else
    if (is_supported_complex_float<eT>::value == true)
      {
      // Call cgssv() overload.
      slu_c::cgssv(options, A, perm_c, perm_r, L, U, B, stat, info);
      }
    else
    if (is_supported_complex_double<eT>::value == true)
      {
      // Call zgssv() overload.
      slu_z::zgssv(options, A, perm_c, perm_r, L, U, B, stat, info);
      }
    }

  } // namespace superlu

#endif
